/*
 * Copyright 2002-2017 Drew Noakes
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 * More information about this project is available at:
 *
 *    https://drewnoakes.com/code/exif/
 *    https://github.com/drewnoakes/metadata-extractor
 */
package com.drew.imaging;

/**
 * Enumeration of supported image file formats.
 */
public enum FileType
{
    Unknown,
    Jpeg,
    Tiff,
    Psd,
    Png,
    Bmp,
    Gif,
    Ico,
    Pcx,
    Riff,

    /** Sony camera raw. */
    Arw,
    /** Canon camera raw, version 1. */
    Crw,
    /** Canon camera raw, version 2. */
    Cr2,
    /** Nikon camera raw. */
    Nef,
    /** Olympus camera raw. */
    Orf,
    /** FujiFilm camera raw. */
    Raf,
    /** Panasonic camera raw. */
    Rw2
}
