/**
 * Contains annotations used to extend the signatures of methods and fields, allowing tools such as IntelliJ IDEA
 * to provide design-time warnings about potential run-time errors.
 */
package com.drew.lang.annotations;
