/*
 * Copyright 2002-2017 Drew Noakes
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 * More information about this project is available at:
 *
 *    https://drewnoakes.com/code/exif/
 *    https://github.com/drewnoakes/metadata-extractor
 */
package com.drew.metadata.gif;

import com.drew.lang.annotations.NotNull;
import com.drew.metadata.TagDescriptor;

/**
 * @author Drew Noakes https://drewnoakes.com
 */
@SuppressWarnings("WeakerAccess")
public class GifHeaderDescriptor extends TagDescriptor<GifHeaderDirectory>
{
    public GifHeaderDescriptor(@NotNull GifHeaderDirectory directory)
    {
        super(directory);
    }

//    @Override
//    public String getDescription(int tagType)
//    {
//        switch (tagType) {
//            case GifHeaderDirectory.TAG_COMPRESSION:
//                return getCompressionDescription();
//            default:
//                return super.getDescription(tagType);
//        }
//    }
}
